//////////////////////////////////////////////////////////////////////////////
// baghiraclient.h
// -------------------
// Baghira window decoration for KDE
// -------------------
// Copyright (c) 2003, 2004 David Johnson
// Please see the header file for copyright and license information.
//////////////////////////////////////////////////////////////////////////////

#include <kconfig.h>
#include <kglobal.h>
#include <kglobalsettings.h>
#include <kimageeffect.h>
#include <klocale.h>
#include <kwin.h>

#include <qbitmap.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qpainter.h>
#include <qtooltip.h>
#include <qsettings.h>

//#include <X11/extensions/shape.h>
//#include <X11/X.h>
#include <X11/Xlib.h>

#include "baghiraclient.h" 
//#include "images.h"
#include "baghiradecobuttons.h"

using namespace Baghira;

static const int BUTTONSIZE = 16;
static const int BUTTONSIZESMALL = 16;
static const int MENUBUTTONWIDTHSMALL = 20;
static const int MENUBUTTONWIDTH = 20;
static const int DECOSIZE = 8;
int TITLESIZE;// = 26;
static const int TITLESIZESMALL = 18;
int HANDLESIZE; // 	 = 16;
static const int MARGIN = 4;


//////////////////////////////////////////////////////////////////////////////
// BaghiraFactory Class                                                      //
//////////////////////////////////////////////////////////////////////////////

bool BaghiraFactory::initialized_ = false;
Baghira::TitlebarType BaghiraFactory::effect_;
Qt::AlignmentFlags BaghiraFactory::titlealign_;
QColor BaghiraFactory::colors_[ ColorTypeCount ][ 2 ];
KPixmap BaghiraFactory::pix_[ PixmapTypeCount ][ 2 ][ 2 ];
int BaghiraFactory::bordersize_ = BorderTiny;
QPixmap BaghiraFactory::ButtonPixmap_[ 2 ][ 5 ][ 3 ][ 2 ];
QPixmap BaghiraFactory::nostalgia_[3][2];
int BaghiraFactory::contrast_;
int BaghiraFactory::_3DImpact_;
int BaghiraFactory::LineImpact_;
bool BaghiraFactory::bgStipple_;
bool BaghiraFactory::comicFrame_;
bool BaghiraFactory::likeTheBottomBar_;
bool BaghiraFactory::shape_ul;
bool BaghiraFactory::shape_ur;
bool BaghiraFactory::shape_ll;
bool BaghiraFactory::shape_lr;
bool BaghiraFactory::allowEasyClosing_;
bool BaghiraFactory::drawIcon_;
int BaghiraFactory::minimumTitleHeight_;
ButtonStyle BaghiraFactory::buttonStyle_;


extern "C" KDecorationFactory* create_factory() {
	return new Baghira::BaghiraFactory();
}

//////////////////////////////////////////////////////////////////////////////
// BaghiraFactory()
// ---------------
// Constructor

BaghiraFactory::BaghiraFactory() {
	readConfig();
	createPixmaps();
	initialized_ = true;
}

//////////////////////////////////////////////////////////////////////////////
// ~BaghiraFactory()
// ----------------
// Destructor

BaghiraFactory::~BaghiraFactory() {
	initialized_ = false;
}

//////////////////////////////////////////////////////////////////////////////
// createDecoration()
// ------------------
// Create the decoration

KDecoration* BaghiraFactory::createDecoration( KDecorationBridge* b ) {
	return new BaghiraClient( b, this );
}

//////////////////////////////////////////////////////////////////////////////
// reset()
// -------
// Reset the handler. Returns true if decorations need to be remade

bool BaghiraFactory::reset( unsigned long changed ) {
	initialized_ = false;
	changed |= readConfig();
	if ( changed & ( SettingColors | SettingDecoration | SettingBorder ) ) {
		createPixmaps();
	}
	initialized_ = true;

	if ( changed & ( SettingColors | SettingDecoration | SettingFont |
	                 SettingButtons | SettingBorder ) ) {
		return true;
	} else {
		resetDecorations( changed );
		return false;
	}
}

//////////////////////////////////////////////////////////////////////////////
// readConfig()
// ------------
// Read in the configuration file

unsigned long BaghiraFactory::readConfig() {

	QSettings settings;
	bgStipple_ = settings.readBoolEntry( "/qt/Baghira/stippleBackground", true
	                                   );
	contrast_ = settings.readNumEntry( "/qt/Baghira/stippleContrast", 3 );
	KConfig config( "kwinbaghirarc" );
	config.setGroup( "General" );

	unsigned long changed = 0;

	// grab settings
	Qt::AlignmentFlags oldalign = titlealign_;
	QString value = config.readEntry( "TitleAlignment", "AlignHCenter" );
	if ( value == "AlignLeft" )
		titlealign_ = Qt::AlignLeft;
	else if ( value == "AlignHCenter" )
		titlealign_ = Qt::AlignHCenter;
	else if ( value == "AlignRight" )
		titlealign_ = Qt::AlignRight;
	if ( oldalign != titlealign_ )
		changed |= SettingFont;
		
	int oldMinTH = minimumTitleHeight_;
	minimumTitleHeight_ = (ButtonStyle) config.readNumEntry( "minimumTitleHeight", 18 );
	if ( oldMinTH != minimumTitleHeight_ ){
		QFontMetrics fm(options()->font(true, false));
		TITLESIZE = QMAX(minimumTitleHeight_, fm.height());
		changed |= SettingFont;
		changed |= SettingDecoration;
		}

	ButtonStyle oldButtons = buttonStyle_;
	buttonStyle_ = (ButtonStyle) config.readNumEntry( "ButtonStyle", 0 );
	if ( oldButtons != buttonStyle_ )
		changed |= SettingDecoration;
		
	bool oldComic = comicFrame_;
	comicFrame_ = config.readBoolEntry( "DrawComicFrame", false );
	if ( oldComic != comicFrame_ )
		changed |= SettingBorder;

	bool oldShUL = shape_ul;
	shape_ul = config.readBoolEntry( "ShapeUL", false );
	if ( oldShUL != shape_ul ){
		changed |= SettingBorder; changed |= SettingDecoration;
		}

	bool oldShUR = shape_ur;
	shape_ur = config.readBoolEntry( "ShapeUR", false );
	if ( oldShUR != shape_ur ){
		changed |= SettingBorder; changed |= SettingDecoration;
		}
		
	bool oldShLL = shape_ll;
	shape_ll = config.readBoolEntry( "ShapeLL", false );
	if ( oldShLL != shape_ll ){
		changed |= SettingBorder; changed |= SettingDecoration;
		}
		
	bool oldShLR = shape_lr;
	shape_lr = config.readBoolEntry( "ShapeLR", false );
	if ( oldShLR != shape_lr ){
		changed |= SettingBorder; changed |= SettingDecoration;
		}

	bool likedTheBottomBar = likeTheBottomBar_;
	likeTheBottomBar_ = config.readBoolEntry( "BigHandler", false );
	if ( likedTheBottomBar != likeTheBottomBar_ )
		changed |= SettingBorder;

	bool allowEasyClosing__ = allowEasyClosing_;
	allowEasyClosing_ = config.readBoolEntry( "allowEasyClosing", false );
	if ( allowEasyClosing__ != allowEasyClosing_ )
		changed |= SettingDecoration;
		
	bool oldDrawIcon_ = drawIcon_;
	drawIcon_ = config.readBoolEntry( "drawIcon", true );
	if ( oldDrawIcon_ = drawIcon_ )
		changed |= SettingFont;
		changed |= SettingDecoration;
		
	Baghira::TitlebarType oldeffect = effect_;
	effect_ = ( Baghira::TitlebarType ) config.readNumEntry( "TitleEffect", Baghira::Gradient );
	if ( oldeffect != effect_ )
		changed |= SettingDecoration;
		
	int old3DImpact = _3DImpact_;
	_3DImpact_ = config.readNumEntry( "3DImpact", 20 );
	if ( old3DImpact != _3DImpact_ )
		changed |= SettingDecoration;

	int oldLineImpact = LineImpact_;
	LineImpact_ = config.readNumEntry( "LineImpact", 40 );
	if ( oldLineImpact != LineImpact_ )
		changed |= SettingDecoration;

	int oldbordersize = bordersize_;
	switch ( options() ->preferredBorderSize( this ) ) {
	case BorderTiny:
		bordersize_ = 0;
		break;
	case BorderNormal:
		bordersize_ = 1;
		break;
	case BorderLarge:
		bordersize_ = 2;
		break;
	case BorderVeryLarge:
		bordersize_ = 4;
		break;
	case BorderHuge:
		bordersize_ = 6;
		break;
	case BorderVeryHuge:
	case BorderOversized:
		bordersize_ = 8;
		break;
	default:
		bordersize_ = 0;
	}
	if ( oldbordersize != bordersize_ )
		changed |= SettingBorder;

	// some colors that KWin doesn't handle
	// Note that KDE doesn't support these, so they will all default
	config.setGroup( "WM" );

	QColor oldcolor = colors_[ TitleBtnBlend ][ true ];
	colors_[ TitleBtnBlend ][ true ] = KDecoration::options() ->color( ColorButtonBg, true );
	colors_[ TitleBtnBlend ][ true ] = config.readColorEntry( "activeTitleBtnBlend", &colors_[ TitleBtnBlend ][ true ] );
	if ( oldcolor != colors_[ TitleBtnBlend ][ true ] )
		changed |= SettingColors;

	oldcolor = colors_[ TitleBtnBlend ][ false ];
	colors_[ TitleBtnBlend ][ false ] = KDecoration::options() ->color( ColorButtonBg, false );
	colors_[ TitleBtnBlend ][ false ] = config.readColorEntry( "inactiveTitleBtnBlend", &colors_[ TitleBtnBlend ][ false ] );
	if ( oldcolor != colors_[ TitleBtnBlend ][ false ] )
		changed |= SettingColors;

	oldcolor = colors_[ TitleBtnText ][ true ];
	bool light = ( qGray( KDecoration::options() ->color( ColorButtonBg, true ).rgb() ) > 128 );

	colors_[ TitleBtnText ][ true ] = light ?
	                                  KDecoration::options() ->color( ColorButtonBg, true ).dark() :
	                                  KDecoration::options() ->color( ColorButtonBg, true ).light();

	colors_[ TitleBtnFg ][ true ] = light ? Qt::black : Qt::white;
	colors_[ TitleBtnFg ][ true ] = config.readColorEntry( "activeTitleBtnFg", &colors_[ TitleBtnFg ][ true ] );
	if ( oldcolor != colors_[ TitleBtnText ][ true ] )
		changed |= SettingColors;

	oldcolor = colors_[ TitleBtnText ][ false ];
	light = ( qGray( KDecoration::options() ->color( ColorButtonBg, false ).rgb() ) > 128 );
	colors_[ TitleBtnText ][ false ] = light ?
	                                   KDecoration::options() ->color( ColorButtonBg, false ).dark() :
	                                   KDecoration::options() ->color( ColorButtonBg, false ).light();

	colors_[ TitleBtnFg ][ false ] = light ? Qt::black : Qt::white;
	colors_[ TitleBtnFg ][ false ] = config.readColorEntry( "inactiveTitleBtnFg", &colors_[ TitleBtnFg ][ false ] );
	if ( oldcolor != colors_[ TitleBtnText ][ false ] )
		changed |= SettingColors;

	return changed;
}

void BaghiraFactory::createOS8Buttons(){
	QPainter p;
	QColor initColor;
	for ( int active = 0; active <= 1; ++active ){
		for (int t = 0; t < 3; t++){
			initColor = QColor(KDecoration::options()->color( ColorTitleBar, active ).dark(100 + t*15));
			nostalgia_[t][active] = QPixmap(BUTTONSIZE, BUTTONSIZE);
			p.begin(&nostalgia_[t][active]);
			p.setPen(KDecoration::options()->color( ColorTitleBar, active ));
			p.drawPoint(0,BUTTONSIZE-1);
			p.drawPoint(BUTTONSIZE-1,0);
			p.setPen(initColor);
			// dark color
			// outer relief ===============================
			p.drawLine(0,0,0,BUTTONSIZE-2);
			p.drawLine(0,0,BUTTONSIZE-2,0);
			// =============================================
			// inner relief ================================
			if (t < 2){
				p.drawLine(3,BUTTONSIZE-3,BUTTONSIZE-3,BUTTONSIZE-3);
				p.drawLine(BUTTONSIZE-3,3,BUTTONSIZE-3,BUTTONSIZE-3);
			}
			else{
				p.drawLine(2,2,2,BUTTONSIZE-4);
				p.drawLine(2,2,BUTTONSIZE-4,2);
			}
			// ========================================
			for (int i = 0; i < BUTTONSIZE - 6; i++){
				if (i == BUTTONSIZE - 7){
					p.drawPoint(2, BUTTONSIZE - 3);
					p.drawPoint(BUTTONSIZE - 3, 2);
				}
				for (int j = 0; j < i+1; j++){
					p.setPen(initColor.light(100 + (i + j)*3));
					p.drawPoint(i+3,j+3);
					if (i != j) p.drawPoint(j+3,i+3);
				}
			}
			// Color is now Bright ;-)
			// inner relief ===============================
			if (t < 2){
				p.drawLine(2,2,2,BUTTONSIZE-4);
				p.drawLine(2,2,BUTTONSIZE-4,2);
			}
			else{
				p.drawLine(3,BUTTONSIZE-3,BUTTONSIZE-3,BUTTONSIZE-3);
				p.drawLine(BUTTONSIZE-3,3,BUTTONSIZE-3,BUTTONSIZE-3);
			}
			// ================================
			// outer relief ======================================
			p.drawLine(1,BUTTONSIZE-1,BUTTONSIZE-1,BUTTONSIZE-1);
			p.drawLine(BUTTONSIZE-1,1,BUTTONSIZE-1,BUTTONSIZE-1);
			//==================================
			// black frame
			p.setPen(Qt::black);
			p.drawRect(1,1,BUTTONSIZE-2,BUTTONSIZE-2);
			p.end();
		}
	}
}

QPixmap &BaghiraFactory::ButtonPixmap( Baghira::WindowState wState, int small,
                                       Baghira::ButtonType type, Baghira::ButtonState state ) {

	// don't ask - you wouldn't want to know ;-)
	if ( type == Baghira::MenuButton ) {
		if ( state == Baghira::ButtonInactive || state == Baghira::ButtonActive )
			state = Baghira::Hovered;
		else if ( state == Baghira::Hovered )
			state = Baghira::Pressed;
	} else if ( ( state == Baghira::ButtonInactive && type != Baghira::HelpButton ) ||
	            type == Baghira::StickyButton ) {
		type = Baghira::MenuButton;
		state = Baghira::ButtonActive;
	} else if ( type == Baghira::HelpButton ) {
		if ( state == Baghira::ButtonInactive )
			state = Baghira::Hovered;
		else if ( state == Baghira::Pressed )
			state = Baghira::ButtonActive;
	}
	// end of mapping

	return ButtonPixmap_[ wState ][ type ][ state ][ small ];
}

QImage* BaghiraFactory::DecoImage( QPixmap *pix, int x, int y, int width, int height ) {
	QPixmap tmpPix( width, height );
	bitBlt ( &tmpPix, 0, 0, pix, x, y, width, height );
	QImage *returnImage = new QImage( tmpPix.convertToImage() );
	return returnImage;
}

//////////////////////////////////////////////////////////////////////////////
// createPixmaps()
// ---------------
// Create all our pixmaps
void BaghiraFactory::createPixmaps() {
	QPainter painter;
	QColorGroup group;

	QImage tmpResult;
	QImage tmpDeco;
	QFont tmpFont;
	QImage buttonImage;
	int B_3 = (int)((BUTTONSIZE -1 )/ 3);
	int B_6 = B_3*2;
	QPainter icyAPainter;
	
	for ( int active = 0; active <= 1; ++active ) {
		//    	group = KDecoration::options()->colorGroup(ColorTitleBar, active);
		for ( int small = 0; small <= 1; ++small ) {
			KPixmap &tbar = pix_[ TitleBar ][ active ][ small ];


			// create gradients and fills
			//	    if (effect_ == Baghira::Gradient || effect_ == Baghira::Stipples /*|| effect_ == Baghira::Glossy * /
			if (effect_ != Baghira::Brushed){
				// resize pixmap
				if ( small ) {
					tbar.resize( 100, TITLESIZESMALL );
				} else {
					tbar.resize( 100, TITLESIZE );
				}
				createGradient( tbar, KDecoration::options() ->color( ColorTitleBar, active ), KDecoration::options() ->color( ColorTitleBlend, active ), effect_ );
			}
			else{
				KPixmap &tcenter = pix_[ TitleCenter ][ active ][ small ];
				QImage brushTile = baghira_findImage( "brushed-tile" );
				QImage brushGradient = baghira_findImage( "brushed-gradient" );
				// resize pixmap
				tbar.resize( 128, 24 );
				tcenter.resize( 512, 24 );
				bitBlt(&tbar, 0, 0, &brushTile, 0, 0, 128, 24);
				bitBlt(&tcenter, 0, 0, &brushGradient, 0, 0, 512, 24);
			}

			switch ( buttonStyle_ ) {
				
			// Load Jaguar Buttons =========================================================================
				
			case Jaguar:
				
				buttonImage = baghira_findImage( "jaguar-min" );
				KImageEffect::blend( buttonImage, tmpDeco = *DecoImage( &tbar, 0, ( tbar.height()
				                     - BUTTONSIZE ) / 2, BUTTONSIZE, BUTTONSIZE ), tmpResult );
				ButtonPixmap_[ active ][ MinButton ][ ButtonActive ][ small ] = QPixmap( tmpResult );

				buttonImage = baghira_findImage( "jaguar-max" );
				KImageEffect::blend( buttonImage, tmpDeco, tmpResult );
				ButtonPixmap_[ active ][ MaxButton ][ ButtonActive ][ small ] = QPixmap( tmpResult );

				buttonImage = baghira_findImage( "jaguar-close" );
				KImageEffect::blend( buttonImage, tmpDeco, tmpResult );
				ButtonPixmap_[ active ][ CloseButton ][ ButtonActive ][ small ] = QPixmap( tmpResult );

				buttonImage = baghira_findImage( "jaguar-min-focus" );
				KImageEffect::blend( buttonImage, tmpDeco = *DecoImage( &tbar, 0, ( tbar.height()
				                     - BUTTONSIZE ) / 2, BUTTONSIZE, BUTTONSIZE ), tmpResult );
				ButtonPixmap_[ active ][ MinButton ][ Hovered ][ small ] = QPixmap( tmpResult );

				buttonImage = baghira_findImage( "jaguar-max-focus" );
				KImageEffect::blend( buttonImage, tmpDeco, tmpResult );
				ButtonPixmap_[ active ][ MaxButton ][ Hovered ][ small ] = QPixmap( tmpResult );

				buttonImage = baghira_findImage( "jaguar-close-focus" );
				KImageEffect::blend( buttonImage, tmpDeco, tmpResult );
				ButtonPixmap_[ active ][ CloseButton ][ Hovered ][ small ] = QPixmap( tmpResult );

				buttonImage = baghira_findImage( "jaguar-min-pressed" );
				KImageEffect::blend( buttonImage, tmpDeco = *DecoImage( &tbar, 0, ( tbar.height()
				                     - BUTTONSIZE ) / 2, BUTTONSIZE, BUTTONSIZE ), tmpResult );
				ButtonPixmap_[ active ][ MinButton ][ Pressed ][ small ] = QPixmap( tmpResult );

				buttonImage = baghira_findImage( "jaguar-max-pressed" );
				KImageEffect::blend( buttonImage, tmpDeco, tmpResult );
				ButtonPixmap_[ active ][ MaxButton ][ Pressed ][ small ] = QPixmap( tmpResult );

				buttonImage = baghira_findImage( "jaguar-close-pressed" );
				KImageEffect::blend( buttonImage, tmpDeco, tmpResult );
				ButtonPixmap_[ active ][ CloseButton ][ Pressed ][ small ] = QPixmap( tmpResult );

				buttonImage = baghira_findImage( "common-help" );
				KImageEffect::blend( buttonImage, tmpDeco, tmpResult );
				ButtonPixmap_[ active ][ HelpButton ][ ButtonActive ][ small ] = QPixmap( tmpResult );

				buttonImage = baghira_findImage( "common-help-focus" );
				KImageEffect::blend( buttonImage, tmpDeco, tmpResult );
				ButtonPixmap_[ active ][ HelpButton ][ Hovered ][ small ] = QPixmap( tmpResult );

				buttonImage = baghira_findImage( "jaguar-inactive" );
				KImageEffect::blend( buttonImage, tmpDeco, tmpResult );
				ButtonPixmap_[ active ][ MenuButton ][ ButtonActive ][ small ] = QPixmap( tmpResult );

				buttonImage = baghira_findImage( "jaguar-menu" );
				KImageEffect::blend( buttonImage, tmpDeco = *DecoImage( &tbar, 0, ( tbar.height()
				                     - BUTTONSIZE ) / 2, MENUBUTTONWIDTH, BUTTONSIZE ), tmpResult );
				ButtonPixmap_[ active ][ MenuButton ][ Hovered ][ small ] = QPixmap( tmpResult );

				buttonImage = baghira_findImage( "jaguar-menu-pressed" );
				KImageEffect::blend( buttonImage, tmpDeco, tmpResult );
				ButtonPixmap_[ active ][ MenuButton ][ Pressed ][ small ] = QPixmap( tmpResult );
				
				break;
				
			// Load Milk Buttons =========================================================================
				
			case Milk:
				
				buttonImage = baghira_findImage( "milk-min" );
				KImageEffect::blend( buttonImage, tmpDeco = *DecoImage( &tbar, 0, ( tbar.height()
				                     - BUTTONSIZE ) / 2, BUTTONSIZE, BUTTONSIZE ), tmpResult );
				ButtonPixmap_[ active ][ MinButton ][ ButtonActive ][ small ] = QPixmap( tmpResult );

				buttonImage = baghira_findImage( "milk-max" );
				KImageEffect::blend( buttonImage, tmpDeco, tmpResult );
				ButtonPixmap_[ active ][ MaxButton ][ ButtonActive ][ small ] = QPixmap( tmpResult );

				buttonImage = baghira_findImage( "milk-close" );
				KImageEffect::blend( buttonImage, tmpDeco, tmpResult );
				ButtonPixmap_[ active ][ CloseButton ][ ButtonActive ][ small ] = QPixmap( tmpResult );

				buttonImage = baghira_findImage( "milk-min-focus" );
				KImageEffect::blend( buttonImage, tmpDeco = *DecoImage( &tbar, 0, ( tbar.height()
				                     - BUTTONSIZE ) / 2, BUTTONSIZE, BUTTONSIZE ), tmpResult );
				ButtonPixmap_[ active ][ MinButton ][ Hovered ][ small ] = QPixmap( tmpResult );

				buttonImage = baghira_findImage( "milk-max-focus" );
				KImageEffect::blend( buttonImage, tmpDeco, tmpResult );
				ButtonPixmap_[ active ][ MaxButton ][ Hovered ][ small ] = QPixmap( tmpResult );

				buttonImage = baghira_findImage( "milk-close-focus" );
				KImageEffect::blend( buttonImage, tmpDeco, tmpResult );
				ButtonPixmap_[ active ][ CloseButton ][ Hovered ][ small ] = QPixmap( tmpResult );

				buttonImage = baghira_findImage( "milk-min-pressed" );
				KImageEffect::blend( buttonImage, tmpDeco = *DecoImage( &tbar, 0, ( tbar.height()
				                     - BUTTONSIZE ) / 2, BUTTONSIZE, BUTTONSIZE ), tmpResult );
				ButtonPixmap_[ active ][ MinButton ][ Pressed ][ small ] = QPixmap( tmpResult );

				buttonImage = baghira_findImage( "milk-max-pressed" );
				KImageEffect::blend( buttonImage, tmpDeco, tmpResult );
				ButtonPixmap_[ active ][ MaxButton ][ Pressed ][ small ] = QPixmap( tmpResult );

				buttonImage = baghira_findImage( "milk-close-pressed" );
				KImageEffect::blend( buttonImage, tmpDeco, tmpResult );
				ButtonPixmap_[ active ][ CloseButton ][ Pressed ][ small ] = QPixmap( tmpResult );

				buttonImage = baghira_findImage( "common-help" );
				KImageEffect::blend( buttonImage, tmpDeco, tmpResult );
				ButtonPixmap_[ active ][ HelpButton ][ ButtonActive ][ small ] = QPixmap( tmpResult );

				buttonImage = baghira_findImage( "common-help-focus" );
				KImageEffect::blend( buttonImage, tmpDeco, tmpResult );
				ButtonPixmap_[ active ][ HelpButton ][ Hovered ][ small ] = QPixmap( tmpResult );

				buttonImage = baghira_findImage( "milk-inactive" );
				KImageEffect::blend( buttonImage, tmpDeco, tmpResult );
				ButtonPixmap_[ active ][ MenuButton ][ ButtonActive ][ small ] = QPixmap( tmpResult );

				buttonImage = baghira_findImage( "milk-menu" );
				KImageEffect::blend( buttonImage, tmpDeco = *DecoImage( &tbar, 0, ( tbar.height()
				                     - BUTTONSIZE ) / 2, MENUBUTTONWIDTH, BUTTONSIZE ), tmpResult );
				ButtonPixmap_[ active ][ MenuButton ][ Hovered ][ small ] = QPixmap( tmpResult );

				buttonImage = baghira_findImage( "milk-menu-pressed" );
				KImageEffect::blend( buttonImage, tmpDeco, tmpResult );
				ButtonPixmap_[ active ][ MenuButton ][ Pressed ][ small ] = QPixmap( tmpResult );
				
				break;
				
			// Create Nostalgia Buttons
			
			case NostalgiaButton:
//				small++;
				createOS8Buttons();
//				QPainter icyAPainter;
				
				ButtonPixmap_[ active ][ MinButton ][ ButtonActive ][ small ] = QPixmap( nostalgia_[1][active] );
				icyAPainter.begin(&ButtonPixmap_[ active ][ MinButton ][ ButtonActive ][ small ]);
				icyAPainter.setPen(Qt::black);
				icyAPainter.drawLine(2,B_3,BUTTONSIZE - 3,B_3);
				icyAPainter.drawLine(2,B_6,BUTTONSIZE - 3,B_6);
				icyAPainter.end();

				ButtonPixmap_[ active ][ MaxButton ][ ButtonActive ][ small ] = QPixmap( nostalgia_[1][active] );
				icyAPainter.begin(&ButtonPixmap_[ active ][ MaxButton ][ ButtonActive ][ small ]);
				icyAPainter.setPen(Qt::black);
				icyAPainter.drawLine(2,B_6,B_6,B_6);
				icyAPainter.drawLine(B_6,2,B_6,B_6);
				icyAPainter.end();

				ButtonPixmap_[ active ][ CloseButton ][ ButtonActive ][ small ] = QPixmap( nostalgia_[1][active] );

				ButtonPixmap_[ active ][ MinButton ][ Hovered ][ small ] = QPixmap( nostalgia_[0][active] );
				icyAPainter.begin(&ButtonPixmap_[ active ][ MinButton ][ Hovered ][ small ]);
				icyAPainter.setPen(Qt::black);
				icyAPainter.drawLine(2,B_3,BUTTONSIZE - 3,B_3);
				icyAPainter.drawLine(2,B_6,BUTTONSIZE - 3,B_6);
				icyAPainter.end();

				ButtonPixmap_[ active ][ MaxButton ][ Hovered ][ small ] = QPixmap( nostalgia_[0][active] );
				icyAPainter.begin(&ButtonPixmap_[ active ][ MaxButton ][ Hovered ][ small ]);
				icyAPainter.setPen(Qt::black);
				icyAPainter.drawLine(2,B_6,B_6,B_6);
				icyAPainter.drawLine(B_6,2,B_6,B_6);
				icyAPainter.end();

				ButtonPixmap_[ active ][ CloseButton ][ Hovered ][ small ] = QPixmap( nostalgia_[0][active] );

				ButtonPixmap_[ active ][ MinButton ][ Pressed ][ small ] = QPixmap( nostalgia_[2][active] );
				icyAPainter.begin(&ButtonPixmap_[ active ][ MinButton ][ Pressed ][ small ]);
				icyAPainter.setPen(Qt::black);
				icyAPainter.drawLine(3,B_3,BUTTONSIZE - 4,B_3);
				icyAPainter.drawLine(3,B_6,BUTTONSIZE - 4,B_6);
				icyAPainter.end();

				ButtonPixmap_[ active ][ MaxButton ][ Pressed ][ small ] = QPixmap( nostalgia_[2][active] );
				icyAPainter.begin(&ButtonPixmap_[ active ][ MaxButton ][ Pressed ][ small ]);
				icyAPainter.setPen(Qt::black);
				icyAPainter.drawLine(2,B_6,B_6,B_6);
				icyAPainter.drawLine(B_6,2,B_6,B_6);
				icyAPainter.end();

				ButtonPixmap_[ active ][ CloseButton ][ Pressed ][ small ] = QPixmap( nostalgia_[2][active] );

				ButtonPixmap_[ active ][ MenuButton ][ ButtonActive ][ small ] = QPixmap( nostalgia_[1][active] );

				ButtonPixmap_[ active ][ MenuButton ][ Hovered ][ small ] = QPixmap( nostalgia_[1][active] );
				icyAPainter.begin(&ButtonPixmap_[ active ][ MenuButton ][ Hovered ][ small ]);
				icyAPainter.setPen(Qt::black);
				icyAPainter.drawLine(2,B_3,BUTTONSIZE-3,B_3);
				icyAPainter.end();

				ButtonPixmap_[ active ][ MenuButton ][ Pressed ][ small ] = QPixmap( nostalgia_[0][active] );
				icyAPainter.begin(&ButtonPixmap_[ active ][ MenuButton ][ Pressed ][ small ]);
				icyAPainter.setPen(Qt::black);
				icyAPainter.drawLine(2,B_3,BUTTONSIZE-3,B_3);
				icyAPainter.end();
				
				tmpFont = QFont(options() ->font( true, false ));
				tmpFont.setPixelSize(BUTTONSIZE - 4);
				
				ButtonPixmap_[ active ][ HelpButton ][ ButtonActive ][ small ] = QPixmap( nostalgia_[1][active] );
				icyAPainter.begin(&ButtonPixmap_[ active ][ HelpButton ][ ButtonActive ][ small ]);
				icyAPainter.setFont(tmpFont);
				icyAPainter.setPen(Qt::black);
				icyAPainter.drawText( 0, 0, BUTTONSIZE, BUTTONSIZE, Qt::AlignHCenter | Qt::AlignVCenter, "?" );
				icyAPainter.end();

				ButtonPixmap_[ active ][ HelpButton ][ Hovered ][ small ] = QPixmap( nostalgia_[0][active] );
				icyAPainter.begin(&ButtonPixmap_[ active ][ HelpButton ][ Hovered ][ small ]);
				icyAPainter.setFont(tmpFont);
				icyAPainter.setPen(Qt::black);
				icyAPainter.drawText( 0, 0, BUTTONSIZE, BUTTONSIZE, Qt::AlignHCenter | Qt::AlignVCenter, "?" );
				icyAPainter.end();
				
				break;
				
			// Load Panther Buttons =========================================================================
				
			case Panther:
			default:

				buttonImage = baghira_findImage( "panther-min" );
				KImageEffect::blend( buttonImage, tmpDeco = *DecoImage( &tbar, 0, ( tbar.height()
				                     - BUTTONSIZE ) / 2, BUTTONSIZE, BUTTONSIZE ), tmpResult );
				ButtonPixmap_[ active ][ MinButton ][ ButtonActive ][ small ] = QPixmap( tmpResult );

				buttonImage = baghira_findImage( "panther-max" );
				KImageEffect::blend( buttonImage, tmpDeco, tmpResult );
				ButtonPixmap_[ active ][ MaxButton ][ ButtonActive ][ small ] = QPixmap( tmpResult );

				buttonImage = baghira_findImage( "panther-close" );
				KImageEffect::blend( buttonImage, tmpDeco, tmpResult );
				ButtonPixmap_[ active ][ CloseButton ][ ButtonActive ][ small ] = QPixmap( tmpResult );

				buttonImage = baghira_findImage( "panther-min-focus" );
				KImageEffect::blend( buttonImage, tmpDeco = *DecoImage( &tbar, 0, ( tbar.height()
				                     - BUTTONSIZE ) / 2, BUTTONSIZE, BUTTONSIZE ), tmpResult );
				ButtonPixmap_[ active ][ MinButton ][ Hovered ][ small ] = QPixmap( tmpResult );

				buttonImage = baghira_findImage( "panther-max-focus" );
				KImageEffect::blend( buttonImage, tmpDeco, tmpResult );
				ButtonPixmap_[ active ][ MaxButton ][ Hovered ][ small ] = QPixmap( tmpResult );

				buttonImage = baghira_findImage( "panther-close-focus" );
				KImageEffect::blend( buttonImage, tmpDeco, tmpResult );
				ButtonPixmap_[ active ][ CloseButton ][ Hovered ][ small ] = QPixmap( tmpResult );

				buttonImage = baghira_findImage( "panther-min-pressed" );
				KImageEffect::blend( buttonImage, tmpDeco = *DecoImage( &tbar, 0, ( tbar.height()
				                     - BUTTONSIZE ) / 2, BUTTONSIZE, BUTTONSIZE ), tmpResult );
				ButtonPixmap_[ active ][ MinButton ][ Pressed ][ small ] = QPixmap( tmpResult );

				buttonImage = baghira_findImage( "panther-max-pressed" );
				KImageEffect::blend( buttonImage, tmpDeco, tmpResult );
				ButtonPixmap_[ active ][ MaxButton ][ Pressed ][ small ] = QPixmap( tmpResult );

				buttonImage = baghira_findImage( "panther-close-pressed" );
				KImageEffect::blend( buttonImage, tmpDeco, tmpResult );
				ButtonPixmap_[ active ][ CloseButton ][ Pressed ][ small ] = QPixmap( tmpResult );

				buttonImage = baghira_findImage( "common-help" );
				KImageEffect::blend( buttonImage, tmpDeco, tmpResult );
				ButtonPixmap_[ active ][ HelpButton ][ ButtonActive ][ small ] = QPixmap( tmpResult );

				buttonImage = baghira_findImage( "common-help-focus" );
				KImageEffect::blend( buttonImage, tmpDeco, tmpResult );
				ButtonPixmap_[ active ][ HelpButton ][ Hovered ][ small ] = QPixmap( tmpResult );

				buttonImage = baghira_findImage( "panther-inactive" );
				KImageEffect::blend( buttonImage, tmpDeco, tmpResult );
				ButtonPixmap_[ active ][ MenuButton ][ ButtonActive ][ small ] = QPixmap( tmpResult );

				buttonImage = baghira_findImage( "panther-menu" );
				KImageEffect::blend( buttonImage, tmpDeco = *DecoImage( &tbar, 0, ( tbar.height()
				                     - BUTTONSIZE ) / 2, MENUBUTTONWIDTH, BUTTONSIZE ), tmpResult );
				ButtonPixmap_[ active ][ MenuButton ][ Hovered ][ small ] = QPixmap( tmpResult );

				buttonImage = baghira_findImage( "panther-menu-pressed" );
				KImageEffect::blend( buttonImage, tmpDeco, tmpResult );
				ButtonPixmap_[ active ][ MenuButton ][ Pressed ][ small ] = QPixmap( tmpResult );

			}
		}
	}
}

//////////////////////////////////////////////////////////////////////////////
// borderSizes()
// -------------
// Return list of valid border sizes

QValueList<BaghiraFactory::BorderSize> BaghiraFactory::borderSizes() const {
	return QValueList<BorderSize>() << BorderTiny << BorderNormal
	       << BorderLarge << BorderVeryLarge
	       << BorderHuge << BorderVeryHuge;
}

//////////////////////////////////////////////////////////////////////////////
// BaghiraButton Class                                                        //
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
// BaghiraButton()
// --------------
// Constructor

BaghiraButton::BaghiraButton( BaghiraClient *parent, const char *name, const QString& tip, bool small, ButtonType type )
		: QButton( parent->widget(), name ), client_( parent ), type_( type ), deco_( 0 ), icon_(), small_( small ), lastmouse_( 0 ), mouseover_( false ) {
	setBackgroundMode( NoBackground );
	setCursor( arrowCursor );
	QToolTip::add( this, tip );
	down_ = false;
	mouseover_ = false;

	if ( small_ ) {
		setFixedSize( type == Baghira::MenuButton && BaghiraFactory::buttonStyle() != NostalgiaButton ? MENUBUTTONWIDTHSMALL :
		              BUTTONSIZESMALL, BUTTONSIZESMALL );
	} else {
		setFixedSize( type == Baghira::MenuButton && BaghiraFactory::buttonStyle() != NostalgiaButton ? MENUBUTTONWIDTH : BUTTONSIZE,
		              BUTTONSIZE );
	}
	for ( int bs = Baghira::ButtonInactive; bs <= Baghira::Pressed; bs++ ) {
		pixmap[ Baghira::WindowInactive ][ ( Baghira::ButtonState ) bs ] =
		    &BaghiraFactory::ButtonPixmap( Baghira::WindowInactive, small, type,
		                                   ( Baghira::ButtonState ) bs );
		pixmap[ Baghira::WindowActive ][ ( Baghira::ButtonState ) bs ] =
		    &BaghiraFactory::ButtonPixmap( Baghira::WindowActive, small, type,
		                                   ( Baghira::ButtonState ) bs );
	}
}

BaghiraButton::~BaghiraButton() {
	if ( deco_ )
		delete deco_;
}


//////////////////////////////////////////////////////////////////////////////
// sizeHint()
// ----------
// Return size hint

QSize BaghiraButton::sizeHint() const {
	if ( small_ )
		return QSize( BUTTONSIZESMALL, BUTTONSIZESMALL );
	else
		return QSize( BUTTONSIZE, BUTTONSIZE );
}

//////////////////////////////////////////////////////////////////////////////
// enterEvent()
// ------------
// Mouse has entered the button

void BaghiraButton::enterEvent( QEvent *e ) {
	mouseover_ = true;
	repaint( false );
	QButton::enterEvent( e );
}

//////////////////////////////////////////////////////////////////////////////
// leaveEvent()
// ------------
// Mouse has left the button

void BaghiraButton::leaveEvent( QEvent *e ) {
	mouseover_ = false;
	if ( type_ == Baghira::MenuButton )
		down_ = false;
	repaint( false );
	QButton::leaveEvent( e );
}

//////////////////////////////////////////////////////////////////////////////
// mousePressEvent()
// -----------------
// Button has been pressed

void BaghiraButton::mousePressEvent( QMouseEvent* e ) {
	lastmouse_ = e->button();

	// translate and pass on mouse event
	int button = LeftButton;
	if ( ( type_ != MaxButton ) && ( e->button() != LeftButton ) ) {
		button = NoButton; // middle & right buttons inappropriate
	}
	down_ = true;
	QMouseEvent me( e->type(), e->pos(), e->globalPos(), button, e->state() );
	repaint( false );
	QButton::mousePressEvent( &me );
}

//////////////////////////////////////////////////////////////////////////////
// mouseReleaseEvent()
// -----------------
// Button has been released

void BaghiraButton::mouseReleaseEvent( QMouseEvent* e ) {
	lastmouse_ = e->button();

	// translate and pass on mouse event
	int button = LeftButton;
	if ( ( type_ != MaxButton ) && ( e->button() != LeftButton ) ) {
		button = NoButton; // middle & right buttons inappropriate
	}
	down_ = false;
	QMouseEvent me( e->type(), e->pos(), e->globalPos(), button, e->state() );
	repaint( false );
	QButton::mouseReleaseEvent( &me );
}

inline ButtonState BaghiraButton::state() {
	if ( down_ )
		return Pressed;
	else if ( mouseover_ )
		return Hovered;
	else if ( client_->isActive() )
		return ButtonActive;
	else
		return ButtonInactive;
}

//////////////////////////////////////////////////////////////////////////////
// drawButton()
// ------------
// Draw the button

void BaghiraButton::drawButton( QPainter *painter ) {
	if ( !BaghiraFactory::initialized() )
		return ;

	int dx, dy;
	bool active = client_->isActive();

	if ( small_ )
		dx = dy = ( TITLESIZESMALL - BUTTONSIZESMALL ) / 2;
	else
		dx = dy = ( TITLESIZE - BUTTONSIZE ) / 2;

	//	painter->drawPixmap(0, 0, *(pixmap[active ? Baghira::WindowActive : Baghira::WindowInactive ][ state() ] ) );
	painter->drawPixmap( 0, 0, BaghiraFactory::ButtonPixmap( ( Baghira::WindowState ) active, small_, type_, state() ) );
	if ( type_ == Baghira::StickyButton && client_->isOnAllDesktops() ) {
		painter->setBrush( Qt::black );
		BaghiraFactory::buttonStyle() == Jaguar ? painter->drawEllipse ( 7, 6, 3, 3 ) : painter->drawEllipse ( 6, 6, 4, 4 );
	}
}

//////////////////////////////////////////////////////////////////////////////
// BaghiraClient Class                                                        //
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
// BaghiraClient()
// -------------
// Constructor

BaghiraClient::BaghiraClient( KDecorationBridge *b, KDecorationFactory *f )
		: KDecoration( b, f ) {
//		myBridge_ = b;
	;
}

BaghiraClient::~BaghiraClient() {
	for ( int n = 0; n < 6; n++ ) {
		if ( button[ n ] )
			delete button[ n ];
	}
}

//////////////////////////////////////////////////////////////////////////////
// init()
// ------
// Real initialization

void BaghiraClient::init() {
	createMainWidget( WResizeNoErase | WRepaintNoErase );
	widget() ->installEventFilter( this );
	widget() ->setBackgroundMode( NoBackground );

	if ( isResizable() && BaghiraFactory::weLikeTheBottomBar()) {
		HANDLESIZE = 16;
		drawHandlerBar = true;
	} else {
		HANDLESIZE = BaghiraFactory::borderSize();
		drawHandlerBar = false;
	}
	
	if ( isTool() ) { // tool windows have tiny titlebars
		titleheight_ = TITLESIZESMALL;
	} else {
		titleheight_ = TITLESIZE;
	}

	// setup layout
	QVBoxLayout *mainlayout = new QVBoxLayout( widget() );
	QHBoxLayout *titlelayout = new QHBoxLayout();
//	QHBoxLayout *windowlayout = new QHBoxLayout();
	windowlayout = new QHBoxLayout();

	titlebar_ = new QSpacerItem( 1, titleheight_, QSizePolicy::Expanding,
	                             QSizePolicy::Fixed );
	handle_ = new QSpacerItem( 1, HANDLESIZE, QSizePolicy::Expanding,
	                           QSizePolicy::Fixed );

	mainlayout->setResizeMode( QLayout::FreeResize );
	mainlayout->addLayout( titlelayout );
	mainlayout->addLayout( windowlayout );
	mainlayout->addItem( handle_ );

	windowlayout->addSpacing( BaghiraFactory::borderSize() );
	if ( isPreview() ) {
		windowlayout->addWidget(
		    new QLabel( i18n( "<b><center>Baghira preview</center></b>" ),
		                widget() ) );
	} else {
		windowlayout->addItem( new QSpacerItem( 0, 18 ) );
	}
	windowlayout->addSpacing( BaghiraFactory::borderSize() );

	leftClose = false;
	rightClose = false;
	performEasyClose = false;
	
	// setup titlebar
	for ( int n = 0; n < 6; n++ )
		button[ n ] = 0;
	plusminus = 1;
	addButtons( titlelayout, options() ->titleButtonsLeft() );
	titlelayout->addItem( titlebar_ );
	plusminus = -1;
//	inactiveOverlay = NULL;
//	qWarning("setting faded FALSE (1)");
	isFaded = false;
	addButtons( titlelayout, options() ->titleButtonsRight() );

	if (BaghiraFactory::effect() == Baghira::Brushed) HandlePix = QPixmap(BaghiraFactory::pix( TitleBar, true, false ));
	else if ( BaghiraFactory::bgStipple() ) {
		HandlePix = QPixmap( HANDLESIZE > 0 ? HANDLESIZE : 1, HANDLESIZE > 4 ? HANDLESIZE : 4);
		QPainter painter;
		painter.begin( &HandlePix );
		painter.fillRect( HandlePix.rect(), widget() ->colorGroup().background() );
		painter.setPen( widget()->colorGroup().background().dark( 100 + BaghiraFactory::contrast() ) );
		for ( int i = 0; i < HandlePix.height(); i += 4 ) {
			painter.drawLine( 0, i, HandlePix.width() - 1, i );
			painter.drawLine( 0, i + 1, HandlePix.width() - 1, i + 1 );
		}
		painter.end();
	}
}

//////////////////////////////////////////////////////////////////////////////
// addButtons()
// ------------
// Add buttons to title layout

void BaghiraClient::addButtons( QBoxLayout *layout, const QString& s ) {
	//    const unsigned char *bitmap;
	QString tip;

	if ( s.length() > 0 ) {
		layout->addSpacing( MARGIN );
		for ( unsigned n = 0; n < s.length(); n++ ) {
			switch ( s[ n ].latin1() ) {
			case 'M':    // Menu button
				if ( !button[ MenuButton ] ) {
					button[ MenuButton ] = new BaghiraButton( this, "menu",
					                       i18n( "Menu" ), isTool(), MenuButton );
					connect( button[ MenuButton ], SIGNAL( pressed() ), this,
					         SLOT( menuButtonPressed() ) );
					layout->addWidget( button[ MenuButton ] );
					if ( n < s.length() - 1 )
						layout->addSpacing( 1 );
					excursion = excursion + plusminus * (MENUBUTTONWIDTH + MARGIN);
				}
				break;

			case 'S':    // Sticky button
				if ( !button[ StickyButton ] ) {
					if ( isOnAllDesktops() ) {
						tip = i18n( "Un-Sticky" );
					} else {
						tip = i18n( "Sticky" );
					}
					button[ StickyButton ] = new BaghiraButton( this, "sticky",
					                         tip, isTool(), StickyButton );
					connect( button[ StickyButton ], SIGNAL( clicked() ), this,
					         SLOT( toggleOnAllDesktops() ) );
					layout->addWidget( button[ StickyButton ] );
					if ( n < s.length() - 1 )
						layout->addSpacing( 1 );
					excursion = excursion + plusminus * (BUTTONSIZE + MARGIN);
				}
				break;

			case 'H':    // Help button
				if ( ( !button[ HelpButton ] ) && providesContextHelp() ) {
					button[ HelpButton ] = new BaghiraButton( this, "help",
					                       i18n( "Help" ), isTool(), HelpButton );
					connect( button[ HelpButton ], SIGNAL( clicked() ), this,
					         SLOT( showContextHelp() ) );
					layout->addWidget( button[ HelpButton ] );
					if ( n < s.length() - 1 )
						layout->addSpacing( 1 );
					excursion = excursion + plusminus * (BUTTONSIZE + MARGIN);
				}
				break;

			case 'I':    // Minimize button
				if ( ( !button[ MinButton ] ) && isMinimizable() ) {
					button[ MinButton ] = new BaghiraButton( this, "iconify",
					                      i18n( "Minimize" ), isTool(), MinButton );
					connect( button[ MinButton ], SIGNAL( clicked() ), this,
					         SLOT( minimize() ) );
					layout->addWidget( button[ MinButton ] );
					if ( n < s.length() - 1 )
						layout->addSpacing( 1 );
					excursion = excursion + plusminus * (BUTTONSIZE + MARGIN);
				}
				break;

			case 'A':    // Maximize button
				if ( ( !button[ MaxButton ] ) && isMaximizable() ) {
					if ( maximizeMode() == MaximizeFull ) {
						tip = i18n( "Restore" );
					} else {
						tip = i18n( "Maximize" );
					}
					button[ MaxButton ] = new BaghiraButton( this, "maximize",
					                      tip, isTool(), MaxButton );
					connect( button[ MaxButton ], SIGNAL( clicked() ), this,
					         SLOT( maxButtonPressed() ) );
					layout->addWidget( button[ MaxButton ] );
					if ( n < s.length() - 1 )
						layout->addSpacing( 1 );
					excursion = excursion + plusminus * (BUTTONSIZE + MARGIN);
				}
				break;

			case 'X':    // Close button
				if ( ( !button[ CloseButton ] ) && isCloseable() ) {
					button[ CloseButton ] = new BaghiraButton( this, "close",
					                        i18n( "Close" ), isTool(), CloseButton );
					connect( button[ CloseButton ], SIGNAL( clicked() ), this,
					         SLOT( closeWindow() ) );
					layout->addWidget( button[ CloseButton ] );
					if ( n < s.length() - 1 )
						layout->addSpacing( 1 );
					excursion = excursion + plusminus * (BUTTONSIZE + MARGIN);
					if (plusminus == 1 && n == 0) // left Button
						leftClose = true;
					if (plusminus == -1 && n == s.length() - 1) // right Button
						rightClose = true;
				}
				break;

			case '_':    // Spacer item
				layout->addSpacing( MARGIN );
				excursion = excursion + plusminus * MARGIN;
			}
		}
		layout->addSpacing( MARGIN );
	}
}

//////////////////////////////////////////////////////////////////////////////
// eventFilter()
// -------------
// Event filter

bool BaghiraClient::eventFilter( QObject *obj, QEvent *e ) {
	if ( obj != widget() /*&& obj != inactiveOverlay*/)
		return false;

	switch ( e->type() ) {
	case QEvent::MouseButtonDblClick: {
			mouseDoubleClickEvent( static_cast<QMouseEvent *>( e ) );
			return true;
		}
	case QEvent::MouseButtonPress: {
//			if (isFaded) removeOverlay();
			if (BaghiraFactory::allowEasyClosing()){
				if (maximizeMode() == MaximizeFull && isActive() && (leftClose && static_cast<QMouseEvent *>( e )->x() == 0 && static_cast<QMouseEvent *>( e )->y() == 0) || (rightClose && static_cast<QMouseEvent *>( e )->x() == ((QWidget *)obj)->width() -1 && static_cast<QMouseEvent *>( e )->y() == 0))
					performEasyClose = true;
				else processMousePressEvent( static_cast<QMouseEvent *>( e ) );
			}
			else processMousePressEvent( static_cast<QMouseEvent *>( e ) );
			return true;
		}
	case QEvent::MouseButtonRelease:{
			if (performEasyClose && maximizeMode() == MaximizeFull && (leftClose && static_cast<QMouseEvent *>( e )->x() == 0 && static_cast<QMouseEvent *>( e )->y() == 0) || (rightClose && static_cast<QMouseEvent *>( e )->x() == ((QWidget *)obj)->width() -1 && static_cast<QMouseEvent *>( e )->y() == 0)){
				closeWindow();
				performEasyClose = false;
			}
			return true;
		}
	case QEvent::Paint: {
			paintEvent( static_cast<QPaintEvent *>( e ) );
			return true;
		}
	case QEvent::Resize: {
			resizeEvent( static_cast<QResizeEvent *>( e ) );
			return true;
		}
	case QEvent::Show: {
			showEvent( static_cast<QShowEvent *>( e ) );
			return true;
		}
	case QEvent::Wheel: {
			wheelEvent(static_cast<QWheelEvent *>( e ));
			return true;
	}
	default: {
			return false;
		}
	}

	return false;
}

//////////////////////////////////////////////////////////////////////////////
// mouseCoubleClickEvent()
// -----------------------
// Doubleclick on title

void BaghiraClient::mouseDoubleClickEvent( QMouseEvent *e ) {
	if ( titlebar_->geometry().contains( e->pos() ) )
		titlebarDblClickOperation();
}

void BaghiraClient::wheelEvent( QWheelEvent *e ) {
	if ( titlebar_->geometry().contains( e->pos() ) && e->state() == Qt::ControlButton ){
		// surf through the desktops :)
		int target, cD;
		if (e->delta() > 0){
			(cD = KWin::currentDesktop()) == KWin::numberOfDesktops() ? target = 1 : target = cD + 1;
		}
		else{
			(cD = KWin::currentDesktop()) == 1 ? target = KWin::numberOfDesktops()  : target = cD - 1;
		}
		setDesktop (target);
		KWin::setCurrentDesktop (target);
	}
}

//////////////////////////////////////////////////////////////////////////////
// paintEvent()
// ------------
// Repaint the window

void BaghiraClient::paintEvent( QPaintEvent* ) {
	if ( !BaghiraFactory::initialized() )
		return ;

	QPainter painter( widget() );
	QColorGroup group;

	QRect title( 0, 0, width(), titlebar_->geometry().height() );
	QRect handle( handle_->geometry() );
	QRect left( 0, title.height(), BaghiraFactory::borderSize(), height() -
	            title.height() - handle.height() );
	QRect right( width() - BaghiraFactory::borderSize(), title.height(),
	             BaghiraFactory::borderSize(), height() - title.height() - handle.height() );

	int x, y, x2, y2, h, w;

	title.rect( &x, &y, &w, &h );
	
	// draw title bar
	if (BaghiraFactory::effect() == Baghira::Brushed){
		if (w > 512){
			int w1 = (w - 512)/2;
			int w2 = w - 512 - w1;
			painter.drawTiledPixmap( 0,0, w1, h, BaghiraFactory::pix( TitleBar, isActive(), isTool() ) );
			painter.drawTiledPixmap( w1, 0, 512, h, BaghiraFactory::pix( TitleCenter, isActive(), isTool() ) );
			painter.drawTiledPixmap( w1 + 512, 0, w2, h, BaghiraFactory::pix( TitleBar, isActive(), isTool() ) );
		}
		else
			painter.drawTiledPixmap( 0, 0, w, h, BaghiraFactory::pix( TitleCenter, isActive(), isTool() ), (512-w)/2, 0 );
	}
	else
		painter.drawTiledPixmap( title, BaghiraFactory::pix( TitleBar, isActive(), isTool() ) );

	x2 = title.right();
	y2 = title.bottom();
	int b = height()-2;
	int shapeW = w;

	const QColor *ttbar = &(options()->color(ColorTitleBar, isActive()));
	group = options() ->colorGroup( ColorTitleBar, isActive() );
	if (BaghiraFactory::drawComicFrame())
		painter.setPen(Qt::black);
	else
		painter.setPen( ttbar->light(100 + BaghiraFactory::_3DImpact()) );
	painter.drawLine( x, y, x2, y );
	
	const QColor *ttblend = &(options()->color(ColorTitleBlend, isActive()));
	
	if (BaghiraFactory::drawComicFrame())
		painter.setPen(Qt::black);
	else
		painter.setPen( ttblend->dark(100 + BaghiraFactory::_3DImpact()) );
	painter.drawLine( x,y,x,y2 );
	painter.drawLine( x2,y,x2,y2 );
	
	if (maximizeMode() != MaximizeFull){
	// draw around the corners ;-)
	if (BaghiraFactory::shapeUL()){
	// Draw edge of top-left corner inside the area removed by the mask.
	painter.drawPoint( 3, 1 );
	painter.drawPoint( 4, 1 );
	painter.drawPoint( 2, 2 );
	painter.drawPoint( 1, 3 );
	painter.drawPoint( 1, 4 );
	}
	if (BaghiraFactory::shapeUR()){
	// Draw edge of top-right corner inside the area removed by the mask.
	painter.drawPoint( shapeW - 5, 1 );
	painter.drawPoint( shapeW - 4, 1 );
	painter.drawPoint( shapeW - 3, 2 );
	painter.drawPoint( shapeW - 2, 3 );
	painter.drawPoint( shapeW - 2, 4 );
	}
	}
//	const QColor *ttblend = &(options()->color(ColorTitleBlend, isActive()));
	painter.setPen( ttblend->dark(100 + BaghiraFactory::LineImpact()) );
	painter.drawLine( x, y2, x2, y2 );

	// draw title text
	titlebar_->geometry().rect( &x, &y, &w, &h );


	painter.setFont( options() ->font( isActive(), isTool() ) );
	painter.setPen( options() ->color( ColorFont, isActive() ) );

	int iconWidth;
	BaghiraFactory::drawIcon() ? iconWidth = 16 : iconWidth = 0;
	
	QRect textBound = painter.boundingRect ( x + 2 * MARGIN + iconWidth, y, w - MARGIN * 2 - iconWidth, h, BaghiraFactory::titleAlign() | AlignVCenter, caption() );
	
	
	int iconLeft = textBound.x() - MARGIN - iconWidth /*- excursion*/;
	if ( iconLeft < x + MARGIN )
		iconLeft = x + MARGIN;

	if (BaghiraFactory::effect() == Baghira::Nostalgia){
	painter.fillRect( iconLeft - 5, y + 1, textBound.right() - iconLeft + 10, h - 2, options()->color( ColorTitleBar, isActive() ) );
	painter.fillRect( title.x(), y + 1, x - title.x()+5, h - 2, options()->color( ColorTitleBar, isActive() ) );
	painter.fillRect( x+w-5, y + 1, title.right() - w + 5, h - 2, options()->color( ColorTitleBar, isActive() ) );
	}
	if (BaghiraFactory::drawIcon()){
		ICONX = iconLeft;
		ICONY = y + ( titleheight_ - 16 ) / 2;
		painter.drawPixmap( ICONX, ICONY, icon().pixmap( QIconSet::Small, QIconSet::Normal ) );
	}

	painter.drawText( x + 2 * MARGIN + iconWidth, y, w - MARGIN * 2 - iconWidth, h, BaghiraFactory::titleAlign() | AlignVCenter, caption() );

	// draw outer frame
	if ( !isShade() ) {
		QColor background = widget() ->colorGroup().background();

		// left side
		left.rect( &x, &y, &w, &h );
		x2 = left.right();
		y2 = left.bottom();
		BaghiraFactory::bgStipple() ? painter.drawTiledPixmap( x, y, w, h, HandlePix ) :
		painter.fillRect( x, y, w, h, background );

		// right side
		right.rect( &x, &y, &w, &h );
		x2 = right.right();
		y2 = right.bottom();
		BaghiraFactory::bgStipple() ? painter.drawTiledPixmap( x, y, w, h, HandlePix ) :
		painter.fillRect( x, y, w, h, background );

	// draw handle =====================================================
	// bottom bar
		// handle middle
		handle.rect( &x, &y, &w, &h );
//		x2 = handle.right() - 15;
		y2 = handle.bottom();

		group = options() ->colorGroup( ColorHandle, isActive() );
		if (BaghiraFactory::effect() == Baghira::Brushed){
			if (w > 512){
				int w1 = (w - 512)/2;
				int w2 = w - 512 - w1;
				painter.drawTiledPixmap( 0, y, w1, h, BaghiraFactory::pix( TitleBar, isActive(), isTool() ) );
				painter.drawPixmap( w1, y, BaghiraFactory::pix( TitleCenter, isActive(), isTool() ) );
				painter.drawTiledPixmap( w1 + 512, y, w2, h, BaghiraFactory::pix( TitleBar, isActive(), isTool() ) );
			}
			else
				painter.drawPixmap( x, y, BaghiraFactory::pix( TitleCenter, isActive(), isTool() ), (512-w)/2, 0, w, h );
		}
		else{
			x += 15;
			w -= 30;
			BaghiraFactory::bgStipple() ? painter.drawTiledPixmap( x, y, w, h, HandlePix ) :
			painter.fillRect( x, y, w, h, background );
		}
		

		//===============================================================

		// handle left =================================================
		// the resize grabber
		x = 0;
		w = 16;
		x2 = 15;

		if ( drawHandlerBar ) {
			group = options() ->colorGroup( ColorFrame, isActive() );

			if (BaghiraFactory::effect() != Baghira::Brushed) painter.fillRect( x, y, w, h, background );

			painter.setPen( group.mid() );
			painter.drawLine( x2, y, x2, y2 );
			painter.drawLine( x, y, x2, y );

			// diagonal
			painter.setPen( group.mid().dark( 130 ) );
			painter.drawLine( x + 2, y + 2, x2 - 2, y2 - 2 );
			painter.drawLine( x + 2, y + 6, x2 - 6, y2 - 2 );
			painter.drawLine( x + 2, y + 10, x2 - 10, y2 - 2 );
		} else {
			BaghiraFactory::bgStipple() ? painter.drawTiledPixmap( x, y, w, h, HandlePix ) :
			painter.fillRect( x, y, w, h, background );
		}

		//===============================================================

		// handle right =================================================
		// the other resize grabber
		x = handle.right() - 15;
		x2 = handle.right();

		if ( drawHandlerBar ) {
			group = options() ->colorGroup( ColorFrame, isActive() );
			if (BaghiraFactory::effect() != Baghira::Brushed) painter.fillRect( x, y, w, h, group.background() );

			// diagonal
			//	painter.setPen(group.mid().dark(130));
			painter.drawLine( x + 2, y2 - 2, x2 - 2, y + 2 );
			painter.drawLine( x + 6, y2 - 2, x2 - 2, y + 6 );
			painter.drawLine( x + 10, y2 - 2, x2 - 2, y + 10 );

			painter.setPen( group.mid() );
			painter.drawLine( x, y, x, y2 );
			painter.drawLine( x, y, x2, y );
		} else {
			BaghiraFactory::bgStipple() ? painter.drawTiledPixmap( x, y, w, h, HandlePix ) :
			painter.fillRect( x, y, w, h, background );
		}

		//===============================================================

		// draw comic frame
		BaghiraFactory::drawComicFrame() ?
			painter.setPen(Qt::black) :
			painter.setPen((BaghiraFactory::effect() != Baghira::Brushed) ? background.dark(100 + BaghiraFactory::_3DImpact()) : ttblend->dark(100 + BaghiraFactory::_3DImpact() * 2));
			
		painter.drawLine( handle.x()/*+7*/, height()-1, x2/*-7*/, height()-1 );

		BaghiraFactory::drawComicFrame() ?
			painter.setPen(Qt::black) :
			painter.setPen((BaghiraFactory::effect() != Baghira::Brushed) ? background.dark(100 + BaghiraFactory::_3DImpact()) : ttblend->dark(100 + BaghiraFactory::_3DImpact()));
				
		
		
		painter.drawLine(0,title.height(),0, height()-1);
		painter.drawLine(width()-1,title.height(),width()-1, height()-1);
		
		
		if (maximizeMode() != MaximizeFull){
		
		int b = height()-2;
		
		if (BaghiraFactory::shapeLL()){
		// Draw edge of bottom-left corner inside the area removed by the mask.
		painter.drawPoint(3, b);
		painter.drawPoint(4, b);
		painter.drawPoint(2, b-1);
		painter.drawPoint(1, b-2);
		painter.drawPoint(1, b-3);
		}
		if (BaghiraFactory::shapeLR()){
		// Draw edge of bottom-right corner inside the area removed by the mask.
		painter.drawPoint(shapeW - 5, b);
		painter.drawPoint(shapeW - 4, b);
		painter.drawPoint(shapeW - 3, b-1);
		painter.drawPoint(shapeW - 2, b-2);
		painter.drawPoint(shapeW - 2, b-3);
		}
		}
	}
}

//////////////////////////////////////////////////////////////////////////////
// resizeEvent()
// -------------
// Window is being resized

void BaghiraClient::resizeEvent( QResizeEvent* ) {
	if ( widget() ->isShown() ) {
		QRegion region = widget() ->rect();
		region = region.subtract( titlebar_->geometry() );
		widget() ->erase( region );
	}
	doShape();
	//    widget()->repaint();
}

/////////////////////////////////////////////////////////////////////////////
// shape Corners
// =) ==================================================================0

void BaghiraClient::doShape() {
	//if (!shape_)
	//  return;

	QRegion mask( 0, 0, width(), height() );

	if (maximizeMode() != MaximizeFull){
	//  QRegion mask(width(), titlebar_->geometry().height());
	int r( width() );
	int b( height()-1 );
	
	if (BaghiraFactory::shapeUL()){
	// Remove top-left corner.
	mask -= QRegion( 0, 0, 5, 1 );
	mask -= QRegion( 0, 1, 3, 1 );
	mask -= QRegion( 0, 2, 2, 1 );
	mask -= QRegion( 0, 3, 1, 2 );
	}
	if (BaghiraFactory::shapeUR()){
	// Remove top-right corner.
	mask -= QRegion( r - 5, 0, 5, 1 );
	mask -= QRegion( r - 3, 1, 3, 1 );
	mask -= QRegion( r - 2, 2, 2, 1 );
	mask -= QRegion( r - 1, 3, 1, 2 );
	}

	if (BaghiraFactory::shapeLL()){
	// Remove bottom-left corner.
	mask -= QRegion(0, b-0, 5, b-1);
	mask -= QRegion(0, b-1, 3, b-1);
	mask -= QRegion(0, b-2, 2, b-1);
	mask -= QRegion(0, b-3, 1, b-2);
	}
	if (BaghiraFactory::shapeLR()){
	// Remove bottom-right corner.
	mask -= QRegion(r - 5, b-0, 5, b-1);
	mask -= QRegion(r - 3, b-1, 3, b-1);
	mask -= QRegion(r - 2, b-2, 2, b-1);
	mask -= QRegion(r - 1, b-3, 1, b-2);
	}
	}
	setMask( mask );
}


//////////////////////////////////////////////////////////////////////////////
// showEvent()
// -----------
// Window is being shown

void BaghiraClient::showEvent( QShowEvent* ) {
	doShape();
	widget() ->repaint();
}

//////////////////////////////////////////////////////////////////////////////
// activeChange()
// --------------
// window active state has changed

void BaghiraClient::activeChange() {

//	if (!isActive()) createOverlay();
//	else if (isFaded) removeOverlay();
	for ( int n = 0; n < 6; n++ )
		if ( button[ n ] )
			button[ n ] ->reset();
	widget() ->repaint( false );
}

//void BaghiraClient::createOverlay(){
/*inactiveOverlay = new QWidget(widget(), 0, WStyle_Customize);
inactiveOverlay->setGeometry(/*geometry().x(), geometry().y()+titleheight_,*\/0,0,geometry().width(), geometry().height()-titleheight_);
inactiveOverlay->installEventFilter( this );
QPixmap fadePixmap = QPixmap::grabWindow(inactiveOverlay->winId(),0,titleheight_,width(),height()-titleheight_);
KPixmapEffect::desaturate( (KPixmap&) fadePixmap, 0.8 );
//inactiveOverlay->setErasePixmap(fadePixmap);
windowlayout->insertWidget(1,inactiveOverlay,1);
//XMapWindow(qt_xdisplay(), inactiveOverlay->winId());
//XReparentWindow(qt_xdisplay(), inactiveOverlay->winId(), widget()->winId(), 0, 0 );
inactiveOverlay->show();
qWarning("setting faded TRUE");
isFaded = true;*/
/*
//stuff for the fade effect
inactiveOverlay = new QWidget(0, 0, WStyle_Customize | WStyle_Splash | WX11BypassWM );
inactiveOverlay->setGeometry(windowlayout->geometry());
fadePixmap = QPixmap::grabWindow(qt_xrootwin(), inactiveOverlay->geometry());
fadePixmap = QPixmap::grabWindow(qt_xrootwin(), inactiveOverlay->geometry());
KPixmapEffect::desaturate( KPixmap & fadePixmap, float desat = 0.3 );
inactiveOverlay->setErasePixmap(fadePixmap);
inactiveOverlay->show(); (fokus???)
oder
XMapWindow(qt_xdisplay(), inactiveOverlay->winId());
*/
//}

//void BaghiraClient::removeOverlay(){
/*	if (inactiveOverlay){
		inactiveOverlay->hide();
//		XUnmapWindow(qt_xdisplay(), inactiveOverlay->winId());
		windowlayout->remove(inactiveOverlay);
		delete inactiveOverlay;
		qWarning("setting faded FALSE (2)");
		isFaded = false;
	}*/
//}

//////////////////////////////////////////////////////////////////////////////
// captionChange()
// ---------------
// The title has changed

void BaghiraClient::captionChange() {
	widget() ->repaint( titlebar_->geometry(), false );
}

//////////////////////////////////////////////////////////////////////////////
// desktopChange()
// ---------------
// Sticky state has changed

void BaghiraClient::desktopChange() {
	bool d = isOnAllDesktops();
	if ( button[ StickyButton ] ) {
		QToolTip::remove
			( button[ StickyButton ] );
		QToolTip::add
			( button[ StickyButton ], d ? i18n( "Un-Sticky" ) : i18n( "Sticky" ) );
	}
}

//////////////////////////////////////////////////////////////////////////////
// iconChange()
// ------------
// The title has changed

void BaghiraClient::iconChange() {
	if (BaghiraFactory::drawIcon()) widget()->repaint(ICONX, ICONY, 16, 16, false);
}

//////////////////////////////////////////////////////////////////////////////
// maximizeChange()
// ----------------
// Maximized state has changed

void BaghiraClient::maximizeChange() {
	bool m = ( maximizeMode() == MaximizeFull );
	if ( button[ MaxButton ] ) {
		QToolTip::remove
			( button[ MaxButton ] );
		QToolTip::add
			( button[ MaxButton ], m ? i18n( "Restore" ) : i18n( "Maximize" ) );
	}
}

//////////////////////////////////////////////////////////////////////////////
// shadeChange()
// -------------
// Called when window shading changes

void BaghiraClient::shadeChange() {
	;
}

//////////////////////////////////////////////////////////////////////////////
// borders()
// ----------
// Get the size of the borders

void BaghiraClient::borders( int &l, int &r, int &t, int &b ) const {
	l = r = BaghiraFactory::borderSize();
	t = titleheight_;
	if ( isShade() ) {
		b = 0;
	} else {
		b = HANDLESIZE;
	}
}

//////////////////////////////////////////////////////////////////////////////
// resize()
// --------
// Called to resize the window

void BaghiraClient::resize( const QSize &size ) {
	widget() ->resize( size );
}

//////////////////////////////////////////////////////////////////////////////
// minimumSize()
// -------------
// Return the minimum allowable size for this decoration

QSize BaghiraClient::minimumSize() const {
	return QSize( 100, 60 );
}

//////////////////////////////////////////////////////////////////////////////
// mousePosition()
// ---------------
// Return mouse position (for resizing)

KDecoration::Position BaghiraClient::mousePosition( const QPoint &point ) const {
	Position pos = PositionCenter;


	if ( point.y() < ( height() - HANDLESIZE ) ) { // inside client
		pos = KDecoration::mousePosition( point );
	} else {
		if ( point.x() >= ( width() - 16 ) )
			pos = PositionBottomRight;
		else if ( point.x() <= 16 )
			pos = PositionBottomLeft;
		else
			pos = PositionBottom;
	}
	return pos;
}


//////////////////////////////////////////////////////////////////////////////
// maxButtonPressed()
// -----------------
// Max button was pressed


void BaghiraClient::maxButtonPressed() {
	if ( button[ MaxButton ] ) {
		switch ( button[ MaxButton ] ->lastMousePress() ) {
		case MidButton:
			if (BaghiraFactory::weLikeTheBottomBar()){
			if ( maximizeMode() == MaximizeVertical ) {
				HANDLESIZE = 16;
				drawHandlerBar = true;
			}
			else {
				HANDLESIZE = BaghiraFactory::borderSize();
				drawHandlerBar = true;
			}
			}
			maximize( maximizeMode() ^ MaximizeVertical );
			//handle_->changeSize(1, HANDLESIZE, QSizePolicy::Expanding, QSizePolicy::Fixed );
			break;
		case RightButton:
			maximize( maximizeMode() ^ MaximizeHorizontal );
			break;
		default:
			if ( maximizeMode() == MaximizeFull ) {
				if (BaghiraFactory::weLikeTheBottomBar()){
				HANDLESIZE = 16;
				drawHandlerBar = true;
				}
				maximize( MaximizeRestore );
				// handle_->changeSize(1, HANDLESIZE, QSizePolicy::Expanding, QSizePolicy::Fixed );
			}
			else {
				HANDLESIZE = BaghiraFactory::borderSize();
				drawHandlerBar = false;
				maximize( MaximizeFull );
			}
		}
	}
}

//////////////////////////////////////////////////////////////////////////////
// menuButtonPressed()
// -------------------
// Menu button was pressed

void BaghiraClient::menuButtonPressed() {
	if ( button[ MenuButton ] ) {
		QPoint pt( button[ MenuButton ] ->rect().bottomLeft().x(),
		           button[ MenuButton ] ->rect().bottomLeft().y() + 4 );
		KDecorationFactory* f = factory();
		showWindowMenu( button[ MenuButton ] ->mapToGlobal( pt ) );
		if ( !f->exists( this ) )
			return ; // we have been destroyed
		button[ MenuButton ] ->setDown( false );
	}
}

//////////////////////////////////////////////////////////////////////////////
// isTool()
// --------
// Is this a tool window?

bool BaghiraClient::isTool() const {
	static const unsigned long winmask = NET::DesktopMask | NET::DialogMask |
	                                     NET::DockMask | NET::MenuMask |
	                                     NET::NormalMask | NET::OverrideMask |
	                                     NET::SplashMask | NET::ToolbarMask |
	                                     NET::TopMenuMask | NET::UtilityMask;

	NET::WindowType type = windowType( winmask );
	return ( type == NET::Menu || type == NET::Toolbar || type == NET::Utility );
}

#include "baghiraclient.moc"
