//////////////////////////////////////////////////////////////////////////////
// baghiraclient.h
// -------------------
// Baghira window decoration for KDE
// -------------------
// Copyright (c) 2003, 2004 David Johnson <david@usermode.org>
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// deal in the Software without restriction, including without limitation the
// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
// sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
// IN THE SOFTWARE.
//////////////////////////////////////////////////////////////////////////////

#ifndef BAGHIRACLIENT_H
#define BAGHIRACLIENT_H

#include <qcolor.h>
#include <qbutton.h>
#include <kdecoration.h>
#include <kdecorationfactory.h>
#include <kpixmap.h>

class QSpacerItem;
class QPoint;

namespace Baghira {


enum PixmapType {
    TitleBar = 0,
    TitleCenter,
//    Handle,
    PixmapTypeCount
};

enum ColorType {
    TitleBtnBlend=0,
    TitleBtnFg,
    TitleBtnText,
    ColorTypeCount
};

enum ButtonType{
	MinButton = 0,
	MaxButton,
	CloseButton,
	MenuButton,
	HelpButton,
	StickyButton // take care !!! button is only virtual!!! (NO PIXMAP!) so don't change order !!!
};

enum ButtonStyle{
	Panther = 0,
	Jaguar,
	Milk,
	NostalgiaButton
};

enum ButtonState{
	ButtonActive,
	Hovered,
	Pressed,
	ButtonInactive
};

enum WindowState{
	WindowInactive,
	WindowActive
};

enum TitlebarType{
	Gradient,
	Stipples,
	Nostalgia,
	Brushed,
	Glossy
};

 class BaghiraClient;

// BaghiraFactory //////////////////////////////////////////////////////////////

class BaghiraFactory: public KDecorationFactory
{
public:
    BaghiraFactory();
    virtual ~BaghiraFactory();
    virtual KDecoration *createDecoration(KDecorationBridge *b);
    virtual bool reset(unsigned long changed);
    virtual QValueList<BorderSize> borderSizes() const;
    static QPixmap &BaghiraFactory::ButtonPixmap(Baghira::WindowState wState, int small, Baghira::ButtonType type, Baghira::ButtonState state);
    QImage* BaghiraFactory::DecoImage(QPixmap *pix, int x, int y, int width, int height);

    static bool initialized();
    static QColor &color(Baghira::ColorType c, bool active=true);
    static KPixmap &pix(PixmapType p, bool active=true, bool small=false);
    static TitlebarType &effect();
    static ButtonStyle buttonStyle();
    static Qt::AlignmentFlags titleAlign();
    static int borderSize();
    static int contrast();
    static int _3DImpact();
    static int LineImpact();
    static bool bgStipple();
    static bool drawComicFrame();
    static bool weLikeTheBottomBar();
    static bool shapeUL();
    static bool shapeUR();
    static bool shapeLL();
    static bool shapeLR();
    static bool drawIcon();
    static bool allowEasyClosing();
    static TitlebarType titleEffect();
//    static KDecorationBridge *bridge();

private:
    unsigned long readConfig();
    void createPixmaps();
    void createOS8Buttons();

private:
    static bool initialized_;
    static bool bgStipple_;
    static bool comicFrame_;
    static bool likeTheBottomBar_;
    static bool shape_ul, shape_ur, shape_ll, shape_lr;
    static bool allowEasyClosing_;
    static bool drawIcon_;
    static ButtonStyle buttonStyle_;
    static TitlebarType effect_;
    static Qt::AlignmentFlags titlealign_;
    static QColor colors_[ColorTypeCount][2];
    static KPixmap pix_[PixmapTypeCount][2][2];
    static QPixmap ButtonPixmap_[2][5][3][2];
    static QPixmap nostalgia_[3][2];
    
    static int contrast_;
    static int bordersize_;
    static int minimumTitleHeight_;
    static int _3DImpact_;
    static int LineImpact_;    
};

inline bool BaghiraFactory::shapeUL()
    { return shape_ul; }
inline bool BaghiraFactory::shapeUR()
    { return shape_ur; }
inline bool BaghiraFactory::shapeLL()
    { return shape_ll; }
inline bool BaghiraFactory::shapeLR()
    { return shape_lr; }

inline bool BaghiraFactory::allowEasyClosing()
    { return allowEasyClosing_; }

inline bool BaghiraFactory::drawIcon()
    { return drawIcon_; }    
        
inline int BaghiraFactory::contrast()
    { return contrast_; }
    
inline int BaghiraFactory::_3DImpact()
    { return _3DImpact_; }
    
inline int BaghiraFactory::LineImpact()
    { return LineImpact_; }

inline bool BaghiraFactory::drawComicFrame()
    { return comicFrame_; }
    
inline bool BaghiraFactory::weLikeTheBottomBar()
    { return likeTheBottomBar_; }
    
inline bool BaghiraFactory::bgStipple()
    { return bgStipple_; }

inline ButtonStyle BaghiraFactory::buttonStyle()
    { return BaghiraFactory::buttonStyle_; } 
    
inline bool BaghiraFactory::initialized()
    { return initialized_; }

inline QColor &BaghiraFactory::color(Baghira::ColorType c, bool active)
    { return colors_[c][active]; }

inline KPixmap &BaghiraFactory::pix(PixmapType p, bool active, bool small)
    { return pix_[p][active][small]; }

inline Baghira::TitlebarType &BaghiraFactory::effect()
    { return effect_; }

inline Qt::AlignmentFlags BaghiraFactory::titleAlign()
    { return titlealign_; }

inline int BaghiraFactory::borderSize()
    { return bordersize_; }

// BaghiraButton ///////////////////////////////////////////////////////////////

class BaghiraButton : public QButton
{
public:
    BaghiraButton(BaghiraClient *parent=0, const char *name=0, const QString &tip=NULL, bool small=false, ButtonType type=MinButton);
    ~BaghiraButton();

    QSize sizeHint() const;
    int lastMousePress() const;
    void reset();
    Baghira::ButtonState state();
//    Baghira::ButtonType type();
inline void setType(Baghira::ButtonType type)
	{type_ = type; }
//inline void setPixmap(Baghira::ButtonState state, QPixmap *pixmap)
//	{pixmap[state] = pixmap;}

private:
    void enterEvent(QEvent *e);
    void leaveEvent(QEvent *e);
    void mousePressEvent(QMouseEvent *e);
    void mouseReleaseEvent(QMouseEvent *e);
    void drawButton(QPainter *painter);

private:
    BaghiraClient *client_;
    ButtonType type_;
    QBitmap *deco_;
    QPixmap *pixmap[2][4];
    KPixmap icon_;
    bool small_;
//    int width;
//    int height;
    int lastmouse_;
    bool mouseover_;
    bool down_;
};

inline int BaghiraButton::lastMousePress() const
    { return lastmouse_; }

inline void BaghiraButton::reset()
    { repaint(false); }
    
// BaghiraClient ///////////////////////////////////////////////////////////////

class BaghiraClient : public KDecoration
{
    Q_OBJECT
public:
    BaghiraClient(KDecorationBridge *b, KDecorationFactory *f);
    virtual ~BaghiraClient();

    virtual void init();

    virtual void activeChange();
    virtual void desktopChange();
    virtual void captionChange();
    virtual void iconChange();
    virtual void maximizeChange();
    virtual void shadeChange();
//    int height() const{
//    	return titleheight_;
//    }

    virtual void borders(int &l, int &r, int &t, int &b) const;
    virtual void resize(const QSize &size);
    virtual QSize minimumSize() const;
    virtual Position mousePosition(const QPoint &point) const;
//    QImage *DecoImage(Baghira::WindowState state, int x, int y, int width, int height);
//    QPixmap *ButtonPixmap(Baghira::ButtonType type, Baghira::WindowState wState, Baghira::ButtonState state, QImage *decoImage);

private:
    void addButtons(QBoxLayout* layout, const QString& buttons);

    bool eventFilter(QObject *obj, QEvent *e);
    void mouseDoubleClickEvent(QMouseEvent *e);
    void wheelEvent(QWheelEvent *e);
    void paintEvent(QPaintEvent *);
    void resizeEvent(QResizeEvent *);
    void showEvent(QShowEvent *);
    bool isTool() const;
    void doShape();
//    void createOverlay();
//    void removeOverlay();

private slots:
    void maxButtonPressed();
    void menuButtonPressed();

private:
    BaghiraButton *button[6];
//    static QPixmap* ButtonPixmap_[2][3][4];
//    static QPixmap* DecoPixmap;
//    static QPixmap* stipples;
//    bool wasActive;
    
    QSpacerItem *titlebar_, *handle_;
    int  titleheight_;
    QPixmap HandlePix;
    bool drawHandlerBar;
    int excursion; // skewness because of unequality of left/right buttons - bad for center aligned titles...
    int plusminus;
    bool isFaded;
    bool leftClose;
    bool rightClose;
    bool performEasyClose;
//    QWidget *inactiveOverlay;
    QHBoxLayout *windowlayout;
    int ICONX, ICONY;
//    KDecorationBridge *myBridge_;
//    QImage* ActiveDecoImage_;
//    QImage* InactiveDecoImage_;
};

} // namespace Baghira

#endif // BAGHIRACLIENT_H
