//////////////////////////////////////////////////////////////////////////////
// baghiraconfig.cc
// -------------------
// Config module for Baghira window decoration
// -------------------
// Copyright (c) 2003, 2004 David Johnson <david@usermode.org>
// Please see the header file for copyright and license information.
//////////////////////////////////////////////////////////////////////////////

#include <kconfig.h>
#include <klocale.h>
#include <kglobal.h>
#include <qbuttongroup.h>
#include <qcombobox.h>
#include <qcheckbox.h>
#include <qgroupbox.h>
#include <qradiobutton.h>
#include <qspinbox.h>
#include <qwhatsthis.h>

#include "baghiraconfig.h"
#include "configdialog.h"
//#include "../images.h"

//////////////////////////////////////////////////////////////////////////////
// BaghiraConfig Class                                                        //
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
// BaghiraConfig()
// -------------
// Constructor

BaghiraConfig::BaghiraConfig(KConfig*, QWidget* parent)
    : QObject(parent), config_(0), dialog_(0)
{
    config_ = new KConfig("kwinbaghirarc");
    KGlobal::locale()->insertCatalogue("kwin_baghira_config");

    dialog_ = new ConfigDialog(parent);

    // load images into titleeffect combo
/*    KPixmap pix;
    config_->setGroup("WM");
    QColor base = config_->readColorEntry("activeBackground");
    for (int n=0; n<EFFECTCOUNT; ++n) {
        pix.resize(100, 22);
        createGradient(pix, base, effects[n]);
        dialog_->titleeffect->insertItem(pix, effects[n]);
    }*/
    
//    dialog_->titleeffect->insertItem("Gradient");
//    dialog_->titleeffect->insertItem("Stipples");

    load(config_);
    dialog_->show();

    // connections
    connect(dialog_->titlealign, SIGNAL(clicked(int)),
            this, SLOT(selectionChanged()));
    connect(dialog_->buttonStyle, SIGNAL(activated(int)),
    	    this, SLOT(selectionChanged()));
    connect(dialog_->titleeffect, SIGNAL(activated(int)), 
            this, SLOT(selectionChanged()));
    connect(dialog_->drawComicFrame, SIGNAL(toggled(bool)),
            this, SLOT(selectionChanged()));
    connect(dialog_->bigHandler, SIGNAL(toggled(bool)),
            this, SLOT(selectionChanged()));
    connect(dialog_->shapeUL, SIGNAL(toggled(bool)),
            this, SLOT(selectionChanged()));
    connect(dialog_->shapeUR, SIGNAL(toggled(bool)),
            this, SLOT(selectionChanged()));
    connect(dialog_->shapeLL, SIGNAL(toggled(bool)),
            this, SLOT(selectionChanged()));
    connect(dialog_->shapeLR, SIGNAL(toggled(bool)),
            this, SLOT(selectionChanged()));
    connect(dialog_->allowEasyClosing, SIGNAL(toggled(bool)),
            this, SLOT(selectionChanged()));
    connect(dialog_->drawIcon, SIGNAL(toggled(bool)),
            this, SLOT(selectionChanged()));    
    connect(dialog_->minTH, SIGNAL(valueChanged(int)),
            this, SLOT(selectionChanged()));
    connect(dialog_->_3DImpact, SIGNAL(valueChanged(int)),
            this, SLOT(selectionChanged()));
    connect(dialog_->LineImpact, SIGNAL(valueChanged(int)),
            this, SLOT(selectionChanged()));

}

//////////////////////////////////////////////////////////////////////////////
// ~BaghiraConfig()
// --------------
// Destructor

BaghiraConfig::~BaghiraConfig()
{
    if (dialog_) delete dialog_;
    if (config_) delete config_;
}

//////////////////////////////////////////////////////////////////////////////
// selectionChanged()
// ------------------
// Selection has changed

void BaghiraConfig::selectionChanged()
{
    emit changed();
}

//////////////////////////////////////////////////////////////////////////////
// load()
// ------
// Load configuration data

void BaghiraConfig::load(KConfig*)
{
    config_->setGroup("General");
    
    QString value = config_->readEntry("TitleAlignment", "AlignHCenter");
    QRadioButton *button = (QRadioButton*)dialog_->titlealign->
        child((const char *)value.latin1());
    if (button) button->setChecked(true);
    
    dialog_->buttonStyle->setCurrentItem(config_->readNumEntry("ButtonStyle",0));
    dialog_->titleeffect->setCurrentItem(config_->readNumEntry("TitleEffect", 0));
    dialog_->drawComicFrame->setChecked(config_->readBoolEntry("DrawComicFrame",false));
    dialog_->bigHandler->setChecked(config_->readBoolEntry("BigHandler",false));
    dialog_->shapeUL->setChecked(config_->readBoolEntry("ShapeUL",true));
    dialog_->shapeUR->setChecked(config_->readBoolEntry("ShapeUR",true));
    dialog_->shapeLL->setChecked(config_->readBoolEntry("ShapeLL",false));
    dialog_->shapeLR->setChecked(config_->readBoolEntry("ShapeLR",false));
    dialog_->allowEasyClosing->setChecked(config_->readBoolEntry("allowEasyClosing",false));
    dialog_->drawIcon->setChecked(config_->readBoolEntry("drawIcon",true));
    dialog_->minTH->setValue(config_->readNumEntry("minimumTitleHeight",18));
    dialog_->_3DImpact->setValue(config_->readNumEntry("3DImpact",20));
    dialog_->LineImpact->setValue(config_->readNumEntry("LineImpact",40));
//     for (int n=0; n<EFFECTCOUNT; ++n) {
// 	if (effects[n] == effectname) {
// 	    dialog_->titleeffect->setCurrentItem(n);
// 	    break;
// 	}
//     }
}

//////////////////////////////////////////////////////////////////////////////
// save()
// ------
// Save configuration data

void BaghiraConfig::save(KConfig*)
{
    config_->setGroup("General");
    QRadioButton *button = (QRadioButton*)dialog_->titlealign->selected();
    if (button) config_->writeEntry("TitleAlignment", QString(button->name()));
    
    config_->writeEntry("ButtonStyle", dialog_->buttonStyle->currentItem());
    config_->writeEntry("TitleEffect", dialog_->titleeffect->currentItem());
    config_->writeEntry("DrawComicFrame", dialog_->drawComicFrame->isChecked());
    config_->writeEntry("BigHandler", dialog_->bigHandler->isChecked());
    config_->writeEntry("ShapeUL", dialog_->shapeUL->isChecked());
    config_->writeEntry("ShapeUR", dialog_->shapeUR->isChecked());
    config_->writeEntry("ShapeLL", dialog_->shapeLL->isChecked());
    config_->writeEntry("ShapeLR", dialog_->shapeLR->isChecked());
    config_->writeEntry("allowEasyClosing", dialog_->allowEasyClosing->isChecked());
    config_->writeEntry("drawIcon", dialog_->drawIcon->isChecked());
    config_->writeEntry("minimumTitleHeight", dialog_->minTH->value());
    config_->writeEntry("3DImpact", dialog_->_3DImpact->value());
    config_->writeEntry("LineImpact", dialog_->LineImpact->value());
    config_->sync();
}

//////////////////////////////////////////////////////////////////////////////
// defaults()
// ----------
// Set configuration defaults

void BaghiraConfig::defaults()
{
    QRadioButton *button = (QRadioButton*)dialog_->titlealign->child("AlignHCenter");
    if (button) button->setChecked(true);
    dialog_->buttonStyle->setCurrentItem(0);
    dialog_->titleeffect->setCurrentItem(0);
    dialog_->drawComicFrame->setChecked(false);
    dialog_->bigHandler->setChecked(false);
    dialog_->shapeUL->setChecked(true);
    dialog_->shapeUR->setChecked(true);
    dialog_->shapeLL->setChecked(false);
    dialog_->shapeLR->setChecked(false);
    dialog_->allowEasyClosing->setChecked(false);
    dialog_->drawIcon->setChecked(true);
    dialog_->minTH->setValue(22);
    dialog_->_3DImpact->setValue(20);
    dialog_->LineImpact->setValue(40);
}

//////////////////////////////////////////////////////////////////////////////
// Plugin Stuff                                                             //
//////////////////////////////////////////////////////////////////////////////

extern "C"
{
    QObject* allocate_config(KConfig* config, QWidget* parent) {
        return(new BaghiraConfig(config, parent));
    }
}

#include "baghiraconfig.moc"
